import Vue from 'vue';
import Vuex from 'vuex';

Vue.use(Vuex);

function findItemByIds(state, checklistId, itemId) {
    let checklist = state.checklists.find(checklist => checklist.id === checklistId);
    if(!checklist) return null;
    return checklist.items.find(item => item.id === itemId);
}

export default new Vuex.Store({
    state: {
        checklists: [{
            id: 1,
            name: 'Wishlist',
            items: [{
                id: 1,
                name: 'TV 50 inches',
                done: false
            }, {
                id: 2,
                name: 'Smartphone',
                done: true
            }, {
                id: 3,
                name: 'Tablet',
                done: false
            }, {
                id: 4,
                name: 'Nintendo Switch',
                done: true
            }]
        }, {
            id: 2,
            name: 'Skills',
            items: [{
                id: 5,
                name: 'Javascript',
                done: true
            }, {
                id: 6,
                name: 'PHP',
                done: false
            }, {
                id: 7,
                name: 'Java',
                done: false
            }, {
                id: 8,
                name: 'Python',
                done: false
            }]
        }, {
            id: 3,
            name: 'Emails',
            items: [{
                id: 9,
                name: 'john@email.com',
                done: false
            }, {
                id: 10,
                name: 'lou@email.com',
                done: false
            }, {
                id: 11,
                name: 'mike@email.com',
                done: true
            }, {
                id: 12,
                name: 'simon@email.com',
                done: false
            }, {
                id: 13,
                name: 'sam@email.com',
                done: true
            }]
        }]
    },
    getters: {
        checklistCount(state) {
            return state.checklists.length;
        },
        completedChecklistCount(state) {
            return state.checklists.filter(checklist => {
                return checklist.items.every(item => item.done);
            }).length;
        },
        itemsCount(state) {
            return state.checklists.reduce((count, checklist) => {
                return count + checklist.items.length;
            }, 0);
        },
        completedItemsCount(state) {
            return state.checklists.reduce((count, checklist) => {
                return checklist.items.reduce((count, item) => {
                    return count + (item.done ? 1 : 0);
                }, count);
            }, 0);
        }
    },
    mutations: {
        toggle(state, payload) {
            let item = findItemByIds(state, payload.checklistId, payload.itemId);
            if(item) {
                item.done = !item.done;
            }
        }
    },
    actions: {
        toggle(context, payload) {
            context.commit('toggle', payload);
        }
    }
});